' ========================================================================================
' Application framework
'
' In your app, define the following:
'Sub AppSetupWin( win as iWindow ) - Finish setting up window (size, position, controls, etc)
'Sub AppCreateDeviceIndependentResources() - Create any required device independent resources
'Sub AppCreateDeviceResources ( ByVal hWnd As Dword ) - Create any device dependent resources
'Sub AppDiscardDeviceResources() - Release any device dependent resources
'Sub AppOnRender( ByVal hWnd As Dword ) - Perform any application specific rendering
'Function AppWindowProc ( ByVal hwnd As Dword, ByVal uMsg As Dword, ByVal wParam As Dword, ByVal lParam As Dword ) As Dword - Handle any events

#Include This Once
%UNICODE=1
#Include Once "windows.inc"
#Include Once "CWindow.inc"
#Include Once "d2d1Helper.inc"
#Include Once "D2DUtils.inc"
#Include Once "D2Dcurion.inc"

' ========================================================================================
' Device Independent Resources
Global helper As ID2D1Helper
Global factory As ID2D1Factory
Global wicFactory As IWICImagingFactory
Global writeFactory As IDWriteFactory
Global paletteNames() As WString
' ========================================================================================

' ========================================================================================
' Device dependent resources
Global renderTarget As ID2D1HwndRenderTarget
Global palettes() As iPalette_c2d
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
Global win As IWindow

Sub FrameworkRun( title As WStringZ, ByVal w As Long, ByVal h As Long, ByVal nCmdShow As Long )
	Local i, explodeSlice As Long

	Randomize
	'Profile "profile.txt"
	' Set process DPI aware
	If AfxGetWindowsVersion => 6 Then SetProcessDPIAware

	' Create an instance of the CWindow class
	win = Class "CWindow"
	If IsNothing( win ) Then Exit Sub

	' Create the application window.
	win.CreateWindow( %NULL, title, 0, 0, 0, 0, 0, 0, CodePtr( FrameworkWindowProc ) )

	' Set the client size
	Call win.SetClientSize( w, h )

	' Center the window
	Call win.CenterWindow()

	Call AppSetupWin( win )

	If IsFalse FrameworkCreateDeviceIndependentResources Then Exit Sub

	' Default message pump ( you can replace it with your own )
	Call win.DoEvents( nCmdShow )
End Sub

' ========================================================================================
' Create resources which are not bound to any device. Their lifetime effectively extends
' for the duration of the app. These resources include the Direct2D, DirectWrite, and
' WIC factories; and a DirectWrite Text Format object ( used for identifying particular
' font characteristics ) and a Direct2D geometry.  WIC ( Windows Imaging Component )
' ========================================================================================
Function FrameworkCreateDeviceIndependentResources() As Long
	Local hr As Long

	' Create a Direct2D factory.
	hr = D2D1CreateFactory2( %D2D1_FACTORY_TYPE_SINGLE_THREADED, factory )
	If hr <> %S_Ok Then Exit Function

	' Create WIC factory. (loading images)
	wicFactory = NewCom ClsId $CLSID_WICImagingFactory
	If IsNothing( wicFactory ) Then Exit Function

	' Create a DirectWrite factory (for text)
	hr = DWriteCreateFactory(%DWRITE_FACTORY_TYPE_SHARED, $IID_IDWriteFactory, writeFactory)
	If hr <> %S_Ok Then Exit Function

	' Create an instance of the CD2D1Helper class
	helper = Class "CD2D1Helper"
	If IsNothing( helper ) Then Exit Function

	PaletteNamesGet_c2d paletteNames()
	Local i As Long
	Local v As String
	For i=0 To UBound( paletteNames() )
		v = paletteNames(i)
	Next
	PalettesAllFromFileLoad_c2d palettes(), helper, factory, paletteNames()

	Call AppCreateDeviceIndependentResources

	Function = %TRUE
End Function

' ========================================================================================
' This method creates resources which are bound to a particular Direct3D device. It's all
' centralized here, in case the resources need to be recreated in case of Direct3D device
' loss ( eg. display change, remoting, removal of video card, etc ).
' ========================================================================================
Function FrameworkCreateDeviceResources ( ByVal hWnd As Dword ) As Dword
	Local hr As Dword
	Local i As Long
	Local rc As RECT
	Local dRender As iDirectRender

	If IsNothing( renderTarget ) Then
		GetClientRect( hWnd, rc )
		hr = factory.CreateHwndRenderTarget( helper.RenderTargetProperties, _
																								helper.HwndRenderTargetProperties( hWnd, helper.SizeU( rc.Right - rc.Left, rc.Bottom - rc.Top ) ), _
																								renderTarget )
		If hr<>%S_Ok Or Not IsObject( renderTarget ) Then
			renderTarget = Nothing
			Exit Function
		End If
		renderTarget.SetAntialiasMode( %D2D1_ANTIALIAS_MODE_ALIASED )

		For i=0 To UBound(palettes())
			dRender = palettes(i)
			If IsObject( dRender ) Then dRender.CreateDeviceResources( renderTarget )
		Next

		Call AppCreateDeviceResources( hWnd )
	End If
	Function = hr
End Function

Sub FrameworkDiscardDeviceResources( hwnd As Dword )
	Local dRender As iDirectRender
	Local i As Long
	renderTarget = Nothing

	Call AppDiscardDeviceResources( hWnd )
	For i=0 To UBound( palettes() )
		dRender = palettes(i)
		If IsObject( dRender ) Then dRender.DiscardDeviceResources()
	Next
End Sub

Function FrameworkOnRender( ByVal hWnd As Dword ) As Dword
	Local hr As Dword
	Local renderTargetSize, bitmapSize As D2D1_SIZE_F

	hr = FrameworkCreateDeviceResources( hWnd )
	If Not IsObject( renderTarget ) Then Exit Function
	If hr = %S_Ok Then
		If ( renderTarget.CheckWindowState And %D2D1_WINDOW_STATE_OCCLUDED ) = 0 Then

			renderTargetSize = renderTarget.GetSize(): ' Retrieve the size of the render target.
			renderTarget.BeginDraw()
			renderTarget.SetAntialiasMode( %D2D1_ANTIALIAS_MODE_PER_PRIMITIVE )
			renderTarget.SetTransform( helper.IdentityMatrix() )
			renderTarget.Clear( helper.ColorF( %BACKR / 255!, %BACKG / 255!, %BACKB / 255! ) )

			Call AppOnRender( hWnd )

			hr = renderTarget.EndDraw()
			If hr = %D2DERR_RECREATE_TARGET Then
				hr = %S_Ok
				Call FrameworkDiscardDeviceResources( hWnd )
			End If
		 End If
	End If
	Function = hr
End Function

Sub FrameworkOnResize( ByVal nWidth As Long, ByVal nHeight As Long )
	Local newSize As D2D1_SIZE_U

	newSize.width = nWidth
	newSize.height = nHeight
	If IsObject( renderTarget ) Then renderTarget.Resize( newSize )
End Sub

Function FrameworkWindowProc ( ByVal hwnd As Dword, ByVal uMsg As Dword, ByVal wParam As Dword, ByVal lParam As Dword ) As Dword
	Local ps As PAINTSTRUCT

	If AppWindowProc( hWnd, uMsg, wParam, lParam ) Then Exit Function

	Select Case uMsg
		Case %WM_EraseBkgnd:  Exit Function

		Case %WM_Size
			Call FrameworkOnResize( Lo( Word, lParam ), Hi( Word, lParam ) )

		Case %WM_Paint
			Call BeginPaint( hWnd, ps )
			Call FrameworkOnRender( hWnd )
			Call EndPaint( hWnd, ps )
			Exit Function

		Case %WM_Destroy
			Call PostQuitMessage(0)
			Exit Function
	End Select

	Function = DefWindowProc( hwnd, uMsg, wParam, lParam )
End Function
